    
--/////////////////////////////////////////////////////////////////////////////
--   ____  ____
--  /   /\/   /
-- /___/  \  /    Vendor: Xilinx
-- \   \   \/     Version : 1.11
--  \   \         Application : Virtex-6 FPGA GTH Transceiver Wizard
--  /   /         Filename : v6_gthwizard_v1_11_xlaui_gth_reset.vhd
-- /___/   /\     Timestamp :
-- \   \  /  \ 
--  \___\/\___\
--
--
-- Module v6_gthwizard_v1_11_xlaui_PULSE_SYNCHRONIZER
-- Generated by Xilinx Virtex-6 FPGA GTH Transceiver Wizard
-- 
-- 
-- (c) Copyright 2009-2012 Xilinx, Inc. All rights reserved.
-- 
-- This file contains confidential and proprietary information
-- of Xilinx, Inc. and is protected under U.S. and
-- international copyright and other intellectual property
-- laws.
-- 
-- DISCLAIMER
-- This disclaimer is not a license and does not grant any
-- rights to the materials distributed herewith. Except as
-- otherwise provided in a valid license issued to you by
-- Xilinx, and to the maximum extent permitted by applicable
-- law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
-- WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
-- AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
-- BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
-- INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
-- (2) Xilinx shall not be liable (whether in contract or tort,
-- including negligence, or under any other theory of
-- liability) for any loss or damage of any kind or nature
-- related to, arising under or in connection with these
-- materials, including for any direct, or any indirect,
-- special, incidental, or consequential loss or damage
-- (including loss of data, profits, goodwill, or any type of
-- loss or damage suffered as a result of any action brought
-- by a third party) even if such damage or loss was
-- reasonably foreseeable or Xilinx had been advised of the
-- possibility of the same.
-- 
-- CRITICAL APPLICATIONS
-- Xilinx products are not designed or intended to be fail-
-- safe, or for use in any application requiring fail-safe
-- performance, such as life-support or safety devices or
-- systems, Class III medical devices, nuclear facilities,
-- applications related to the deployment of airbags, or any
-- other applications that could lead to death, personal
-- injury, or severe property or environmental damage
-- (individually and collectively, "Critical
-- Applications"). Customer assumes the sole risk and
-- liability of any use of Xilinx products in Critical
-- Applications, subject only to applicable laws and
-- regulations governing limitations on product liability.
-- 
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
-- PART OF THIS FILE AT ALL TIMES. 


library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;

--***************************** Entity Declaration ****************************

entity v6_gthwizard_v1_11_xlaui_PULSE_SYNCHRONIZER is
port
(
    SRCCLK_IN                   : in  std_logic;
    DSTCLK_IN                   : in  std_logic;
    SRCPULSE_IN                 : in  std_logic;
    PULSETYPE_IN                : in  std_logic;
    DSTPULSE_OUT                : out std_logic
);

end v6_gthwizard_v1_11_xlaui_PULSE_SYNCHRONIZER;

architecture RTL of v6_gthwizard_v1_11_xlaui_PULSE_SYNCHRONIZER is

--***********************************Parameter Declarations********************

    constant DLY : time := 1 ns;

--******************************Register Declarations**************************
    signal src_pulse_d1         :   std_logic := '0';
    signal src_pulse_d2         :   std_logic := '0';
    signal dst_pulse_d1         :   std_logic := '0';
    signal dst_pulse_d2         :   std_logic := '0';
    signal dst_pulse_d3         :   std_logic := '0';

    signal src_pulse_i          :   std_logic;
    signal pulse_type_i         :   std_logic;
    signal dst_pulse_w1         :   std_logic;

begin
    src_pulse_i  <= SRCPULSE_IN ;
    pulse_type_i <= PULSETYPE_IN;    
    -- -----------------------------
    -- pulse to level source clk
    -- -----------------------------
    process(SRCCLK_IN)
    begin
        if(SRCCLK_IN'event and SRCCLK_IN = '1') then
           src_pulse_d1 <= src_pulse_i after DLY;
           if(pulse_type_i = '1')  then
              if((src_pulse_d1 and not src_pulse_i) = '1') then
                 src_pulse_d2 <= not src_pulse_d2 after DLY;
              end if;
           else
              if((not src_pulse_d1 and src_pulse_i) = '1') then
                 src_pulse_d2 <= not src_pulse_d2 after DLY;
              end if;
           end if;
        end if;
    end process;

    -- -----------------------------
    -- Double flop logic for synchronization
    -- -----------------------------
    process(DSTCLK_IN)
    begin
        if(DSTCLK_IN'event and DSTCLK_IN = '1') then
           dst_pulse_d1 <= src_pulse_d2 after DLY;
           dst_pulse_d2 <= dst_pulse_d1 after DLY;
           dst_pulse_d3 <= dst_pulse_d2 after DLY;
        end if;
    end process;
        
    -- -----------------------------
    -- pulse generation at destination clk
    -- -----------------------------
    dst_pulse_w1 <= dst_pulse_d2 xor dst_pulse_d3;

    DSTPULSE_OUT <= not dst_pulse_w1 when(pulse_type_i = '1') else dst_pulse_w1;
    -- ----------------------------        
end RTL;

