--
-- Copyright (c) 2015 Aitia International Inc.
--
-- See the file "/doc/license.txt" for information on usage and redistribution
-- of this file, and for a DISCLAIMER OF ALL WARRANTIES.
-- 

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_UNSIGNED.all;
USE work.tuw.all;

PACKAGE xlgpcspma_v6 IS

COMPONENT ETH_40G_v6 IS
	PORT(
	-- GTH reference clocks
	REFCLK_N					: IN  STD_LOGIC;
	REFCLK_P					: IN  STD_LOGIC;
	-- DRP clock
	DRP_CLK					: IN  STD_LOGIC;
--	XLG_ExtClk				: IN  STD_LOGIC;
	-- GTH HS pins
	RXN_GTH					: IN  STD_LOGIC_VECTOR(4-1 downto 0);
	RXP_GTH					: IN  STD_LOGIC_VECTOR(4-1 downto 0);
	TXN_GTH					: OUT STD_LOGIC_VECTOR(4-1 downto 0);
	TXP_GTH					: OUT STD_LOGIC_VECTOR(4-1 downto 0);
	-- 2 clocks 4 MAC (156.25MHz)
	XLG_Clk_RX				: OUT STD_LOGIC;
	XLG_Clk_TX				: OUT STD_LOGIC;
	-- Rx MAC
	XLG_RXData				: OUT STD_LOGIC_VECTOR(5*64-1 downto 0);
	XLG_RXMod40				: OUT STD_LOGIC_VECTOR(6-1 downto 0);	
	XLG_RXDv					: OUT STD_LOGIC;
	XLG_RXSof					: OUT STD_LOGIC;
	XLG_RXEof					: OUT STD_LOGIC;
	XLG_RXErr					: OUT STD_LOGIC;	
	XLG_RXCRCErr				: OUT STD_LOGIC;	
	-- Tx MAC
	XLG_TXAv					: OUT STD_LOGIC;
	XLG_TXData				: IN  STD_LOGIC_VECTOR(5*64-1 downto 0);
	XLG_TXMod40				: IN  STD_LOGIC_VECTOR(6-1 downto 0);	
	XLG_TXDv					: IN  STD_LOGIC;
	XLG_TXSof					: IN  STD_LOGIC;
	XLG_TXEof					: IN  STD_LOGIC;
	-- Etc
	PCS_RXSof 				: OUT STD_LOGIC;
	XLG_Rst					: IN  STD_LOGIC;
	XLG_CDR_Rst				: IN  STD_LOGIC;
	XLG_signal_ok				: IN  STD_LOGIC;
	XLG_initdone				: OUT STD_LOGIC;	
	XLG_aligned				: OUT STD_LOGIC;
	XLG_locked66b 				: OUT STD_LOGIC
	);
END COMPONENT;

END PACKAGE;

library IEEE;
use IEEE.std_logic_1164.all;
USE IEEE.std_logic_UNSIGNED.ALL;
use work.tuw.all;
USE work.pcs_phy_40G_pkg.all;

library unisim;
use unisim.vcomponents.all;

ENTITY ETH_40G_v6 IS
	PORT(
	-- GTH reference clocks
	REFCLK_N					: IN  STD_LOGIC;
	REFCLK_P					: IN  STD_LOGIC;
	-- DRP clock
	DRP_CLK					: IN  STD_LOGIC;
--	XLG_ExtClk				: IN  STD_LOGIC;
	-- GTH HS pins
	RXN_GTH					: IN  STD_LOGIC_VECTOR(4-1 downto 0);
	RXP_GTH					: IN  STD_LOGIC_VECTOR(4-1 downto 0);
	TXN_GTH					: OUT STD_LOGIC_VECTOR(4-1 downto 0);
	TXP_GTH					: OUT STD_LOGIC_VECTOR(4-1 downto 0);
	-- 2 clocks 4 MAC (156.25MHz)
	XLG_Clk_RX				: OUT STD_LOGIC;
	XLG_Clk_TX				: OUT STD_LOGIC;
	-- Rx MAC
	XLG_RXData				: OUT STD_LOGIC_VECTOR(5*64-1 downto 0);
	XLG_RXMod40				: OUT STD_LOGIC_VECTOR(6-1 downto 0);	
	XLG_RXDv					: OUT STD_LOGIC;
	XLG_RXSof					: OUT STD_LOGIC;
	XLG_RXEof					: OUT STD_LOGIC;
	XLG_RXErr					: OUT STD_LOGIC;
	XLG_RXCRCErr				: OUT STD_LOGIC;	
	-- Tx MAC
	XLG_TXAv					: OUT STD_LOGIC;
	XLG_TXData				: IN  STD_LOGIC_VECTOR(5*64-1 downto 0);
	XLG_TXMod40				: IN  STD_LOGIC_VECTOR(6-1 downto 0);	
	XLG_TXDv					: IN  STD_LOGIC;
	XLG_TXSof					: IN  STD_LOGIC;
	XLG_TXEof					: IN  STD_LOGIC;
	-- Etc
	PCS_RXSof 				: OUT STD_LOGIC;
	XLG_Rst					: IN  STD_LOGIC;
	XLG_CDR_Rst				: IN  STD_LOGIC;
	XLG_signal_ok				: IN  STD_LOGIC;
	XLG_initdone				: OUT STD_LOGIC;	
	XLG_aligned				: OUT STD_LOGIC;
	XLG_locked66b 				: OUT STD_LOGIC	
	);
END ETH_40G_v6;

ARCHITECTURE ETH_40G_v6_arch OF ETH_40G_v6 IS

COMPONENT CORE_40G IS
	PORT(
	-- 2 double rate clocks 4 MAC (312.5MHz)
	Clk_GTH_Rx 				: IN  STD_LOGIC;
	Clk_GTH_Tx 				: IN  STD_LOGIC;
	Clk_MAC_RX				: IN  STD_LOGIC;
	Clk_MAC_TX				: IN  STD_LOGIC;
	DIN_GTH					: OUT STD_LOGIC_VECTOR(4*64-1 downto 0);
	DOUT_GTH					: IN  STD_LOGIC_VECTOR(4*64-1 downto 0);
	-- Rx MAC
	XLG_RXData				: OUT STD_LOGIC_VECTOR(5*64-1 downto 0);
	XLG_RXMod40				: OUT STD_LOGIC_VECTOR(6-1 downto 0);	
	XLG_RXDv					: OUT STD_LOGIC;
	XLG_RXSof					: OUT STD_LOGIC;
	XLG_RXEof					: OUT STD_LOGIC;
	XLG_RXCRCErr				: OUT STD_LOGIC;	
	XLG_RXErr					: OUT STD_LOGIC;	
	-- Tx MAC
	XLG_TXAv					: OUT STD_LOGIC;
	XLG_TXData				: IN  STD_LOGIC_VECTOR(5*64-1 downto 0);
	XLG_TXMod40				: IN  STD_LOGIC_VECTOR(6-1 downto 0);	
	XLG_TXDv					: IN  STD_LOGIC;
	XLG_TXSof					: IN  STD_LOGIC;
	XLG_TXEof					: IN  STD_LOGIC;
	-- Etc
	PCS_RXSof 				: OUT STD_LOGIC;
	XLG_Rst					: IN  STD_LOGIC;
	XLG_signal_ok				: IN  STD_LOGIC;
	XLG_locked66b				: OUT STD_LOGIC;
	XLG_aligned				: OUT STD_LOGIC
--
	);
END COMPONENT;

SIGNAL DIN_GTH					: STD_LOGIC_VECTOR(4*64-1 downto 0);
SIGNAL DOUT_GTH				: STD_LOGIC_VECTOR(4*64-1 downto 0);
SIGNAL SLIP_GTH				: STD_LOGIC_VECTOR(4-1 downto 0);

SIGNAL clk_GTH_rx				: STD_LOGIC;
SIGNAL clk_GTH_tx				: STD_LOGIC;
SIGNAL clk_MAC_RX				: STD_LOGIC;
SIGNAL clk_MAC_TX				: STD_LOGIC;

SIGNAL dCDR_Rst 				: STD_LOGIC;
SIGNAL ddCDR_Rst 				: STD_LOGIC;
SIGNAL CDR_Rst_imp 				: STD_LOGIC;

attribute keep:string;

attribute keep of clk_MAC_RX :signal is "true";
attribute keep of clk_MAC_TX :signal is "true";
attribute keep of clk_GTH_rx :signal is "true";
attribute keep of clk_GTH_tx :signal is "true";

BEGIN

--
---- 0. MAC clock synthesis (312.5MHz)
--

XLG_clk_RX	<= clk_MAC_RX;
XLG_clk_TX	<= clk_MAC_TX;


--
---- 1. Physical GTH instantiation (1 GTH / 4 lanes)
--
UGTH: pcs_phy_40G
	PORT MAP(
	-- GTH reference clocks
	Q0_REFCLK_N				=> REFCLK_N, --: IN  STD_LOGIC;
	Q0_REFCLK_P				=> REFCLK_P, --: IN  STD_LOGIC;
--	-- DRP clock
	DRP_CLK					=> DRP_CLK, --: IN  STD_LOGIC;
	-- GTH HS pins
	RXN_GTH					=> RXN_GTH, --: IN  STD_LOGIC_VECTOR(11 downto 0);
	RXP_GTH					=> RXP_GTH, --: IN  STD_LOGIC_VECTOR(11 downto 0);
	TXN_GTH					=> TXN_GTH, --: OUT STD_LOGIC_VECTOR(11 downto 0);
	TXP_GTH					=> TXP_GTH, --: OUT STD_LOGIC_VECTOR(11 downto 0)
	-- 1+1 recovered common PCS clocks (156.25MHz RX/TX)
	clk_GTH_rx				=> clk_GTH_rx, --: OUT STD_LOGIC;
	clk_GTH_tx				=> clk_GTH_tx, --: OUT STD_LOGIC;
	-- 2 clocks 4 MAC (156.25MHz)
	clk_MAC_RX				=> clk_MAC_RX, --: OUT STD_LOGIC;
	clk_MAC_TX				=> clk_MAC_TX, --: OUT STD_LOGIC;
	-- RAW Data Output 4 every phys lane
	DOUT_GTH					=> DOUT_GTH, --: OUT STD_LOGIC_VECTOR(10*64-1 downto 0);
	-- RAW Data Input 4 every phys lane
	DIN_GTH					=> DIN_GTH, --: IN  STD_LOGIC_VECTOR(4*64-1 downto 0);
	-- Misc
	CDR_RST 					=> CDR_Rst_imp,
	GTH_INITDONE				=> XLG_initdone
);

UCDR_Rstimp: PROCESS(clk_GTH_tx)
BEGIN
IF (clk_GTH_tx'event AND clk_GTH_tx='1') THEN
	dCDR_Rst		<= XLG_CDR_Rst;
	ddCDR_Rst		<= dCDR_Rst;
	CDR_Rst_imp	<= dCDR_Rst AND (NOT ddCDR_Rst); -- Rise detector
END IF;
END PROCESS;


--
---- 2. 40G PCS/PMA + MAC
--
U40G: CORE_40G
	PORT MAP(
	-- 2 double rate clocks 4 MAC (312.5MHz)
	Clk_GTH_Rx 				=> clk_GTH_rx, --: IN  STD_LOGIC;
	Clk_GTH_Tx 				=> clk_GTH_tx, --: IN  STD_LOGIC;
	Clk_MAC_RX				=> clk_MAC_rx, --: IN  STD_LOGIC;
	Clk_MAC_TX				=> clk_MAC_tx, --: IN  STD_LOGIC;
	DIN_GTH					=> DIN_GTH, --: OUT STD_LOGIC_VECTOR(4*64-1 downto 0);
	DOUT_GTH					=> DOUT_GTH, --: IN  STD_LOGIC_VECTOR(4*64-1 downto 0);
	-- Rx MAC
	XLG_RXData				=> XLG_RXData, --: OUT STD_LOGIC_VECTOR(5*64-1 downto 0);
	XLG_RXMod40				=> XLG_RXMod40, --: OUT STD_LOGIC_VECTOR(6-1 downto 0);	
	XLG_RXDv					=> XLG_RXDv, --: OUT STD_LOGIC;
	XLG_RXSof					=> XLG_RXSof, --: OUT STD_LOGIC;
	XLG_RXEof					=> XLG_RXEof, --: OUT STD_LOGIC;
	XLG_RXCRCErr				=> XLG_RXCRCErr, --: OUT STD_LOGIC;	
	XLG_RXErr					=> XLG_RXErr, --: OUT STD_LOGIC;	
	-- Tx MAC
	XLG_TXAv					=> XLG_TXAv, --: OUT STD_LOGIC;
	XLG_TXData				=> XLG_TXData, --: IN  STD_LOGIC_VECTOR(5*64-1 downto 0);
	XLG_TXMod40				=> XLG_TXMod40, --: IN  STD_LOGIC_VECTOR(6-1 downto 0);	
	XLG_TXDv					=> XLG_TXDv, --: IN  STD_LOGIC;
	XLG_TXSof					=> XLG_TXSof, --: IN  STD_LOGIC;
	XLG_TXEof					=> XLG_TXEof, --: IN  STD_LOGIC;
	-- Etc
	PCS_RXSof 				=> PCS_RXSof, --: OUT STD_LOGIC;
	XLG_Rst					=> XLG_Rst, --: IN  STD_LOGIC;
	XLG_signal_ok				=> XLG_signal_ok, --: IN  STD_LOGIC;
	XLG_locked66b				=> XLG_locked66b, --: OUT STD_LOGIC;
	XLG_aligned				=> XLG_aligned --: OUT STD_LOGIC
	);

END ETH_40G_v6_arch;

